/*
 * \brief  C-API for Genode TSC probe utility
 * \author Josef Soentgen
 * \date   2022-12-08
 */

/*
 * Copyright (C) 2022 Genode Labs GmbH
 *
 * This file is distributed under the terms of the GNU General Public License
 * version 2.
 */

/* Genode includes */
#include <base/log.h>
#include <util/reconstructible.h>

/* local includes */
#include "genode_tsc_probe.h"


enum { MAX_PROBES = 32 };

using Log_tsc_probe = Genode::Log_tsc_probe<Genode::Log::Log_fn>;
static Genode::Constructible<Log_tsc_probe::Stats> _probe_stats[MAX_PROBES];
static Genode::Constructible<Log_tsc_probe>        _probes[MAX_PROBES];


void *genode_log_tsc_stats(unsigned id, unsigned number)
{
	if (id > MAX_PROBES)
		return nullptr;

	if (!_probe_stats[id].constructed())
		_probe_stats[id].construct(number);

	return &*_probe_stats[id];
}


void genode_log_tsc_init(void *p, unsigned id, char const *name)
{
	if (!p)
		return;

	if (id > MAX_PROBES) {
		Genode::error("too many probes, limited to ", (unsigned)MAX_PROBES);
		return;
	}

	Log_tsc_probe::Stats &stats = *reinterpret_cast<Log_tsc_probe::Stats*>(p);

	if (!_probes[id].constructed())
		_probes[id].construct(stats, name);
}


void genode_log_tsc_destroy(unsigned id)
{
	if (id > MAX_PROBES)
		return;

	_probes[id].destruct();
	_probe_stats[id].destruct();
}
